package cz.davidkabelka.myapplication;

import android.Manifest;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.job.JobParameters;
import android.app.job.JobService;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.BitmapFactory;
import android.icu.util.Calendar;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;

import androidx.core.app.ActivityCompat;
import androidx.core.app.NotificationCompat;
import androidx.core.app.NotificationManagerCompat;

public class DbUpdateJobService extends JobService implements AsyncResponse, LocationListener {

    String url = "http://zebramapruhy.asuscomm.com:8086/query?u=telefon&p=telefon_heslo&db=gps_data&q=SELECT%20LAST(latitude),lontitude%20FROM%20gps_poz";

    LocationManager mLocationManager;
    MyAsyncTask asyncTask = new MyAsyncTask();
    int hranice;

    @Override
    public boolean onStartJob(JobParameters jobParameters) {

        hranice = jobParameters.getExtras().getInt("hranice");

        asyncTask.delegate = this;

        //execute the async task
        asyncTask.execute();

        return false;
    }

    @Override
    public void processFinish(String output) {
        //Here you will receive the result fired from async class
        //of onPostExecute(result) method.
        double[] LatLon = poloha();
        Log.e("ERR_Job vysledekDoublu", LatLon[0] + " and " +LatLon[1]);





        String[] latlong = output.split(",");

        double DBlatitude = Double.parseDouble(latlong[1]);
        double DBlongitude = Double.parseDouble(latlong[2]);

        double latitude = LatLon[0];
        double longtitude = LatLon[1];



        Log.e("ERR_Job prehled pozic", "DB " + String.valueOf(DBlatitude)+"  "+String.valueOf(DBlongitude) + " Telefon "+String.valueOf(latitude) +"  "+ String.valueOf(longtitude));
        Log.e("ERR_Job Vypocitana vzdalenost: ", distance(DBlatitude, latitude, DBlongitude, longtitude) + " metru");
        createNotificationChannel();


        Log.e("ERR_Job hranice je ", String.valueOf(hranice));
        if (hranice<distance(DBlatitude, latitude, DBlongitude, longtitude)){
            sendNotification(distance(DBlatitude, latitude, DBlongitude, longtitude));
        }


    }

    public double[] poloha(){
        double[] LatLon= new double[2];


        mLocationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);


        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // TODO: Consider calling
            //    ActivityCompat#requestPermissions
            // here to request the missing permissions, and then overriding
            //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
            //                                          int[] grantResults)
            // to handle the case where the user grants the permission. See the documentation
            // for ActivityCompat#requestPermissions for more details.
            Log.e("ERR_Job", " Chybí oprávnění ");


            return LatLon;
        }
        Location location = mLocationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
        if (location != null) {
            // Do something with the recent location fix
            //  otherwise wait for the update below
            Log.e("ERR_Job", " Poloha získána ");

        } else {
            mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);

            Log.e("ERR_Job", " Něco se rozbilo ");
        }

        LatLon[0] = round(location.getLatitude(),6);
        LatLon[1] = round(location.getLongitude(),6);


        return LatLon;
    }
    public static double round(double value, int places) {
        if (places < 0) throw new IllegalArgumentException();

        long factor = (long) Math.pow(10, places);
        value = value * factor;
        long tmp = Math.round(value);
        return (double) tmp / factor;
    }

    public static double distance(double lat1, double lat2, double lon1,
                                  double lon2) {

        final int R = 6371; // Radius of the earth

        double latDistance = Math.toRadians(lat2 - lat1);
        double lonDistance = Math.toRadians(lon2 - lon1);
        double a = Math.sin(latDistance / 2) * Math.sin(latDistance / 2)
                + Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2))
                * Math.sin(lonDistance / 2) * Math.sin(lonDistance / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        double distance = R * c * 1000; // convert to meters



        distance = Math.pow(distance, 2);

        return Math.sqrt(distance);
    }



    @Override
    public boolean onStopJob(JobParameters jobParameters) {
        return false;
    }

    public void sendNotification(double distance) {

        NotificationCompat.Builder builder = new NotificationCompat.Builder(this,"Notifikace");

        builder.setContentTitle("Vozidlo je vzdáleno");
        builder.setContentText(round(distance,2) + " metrů");
        builder.setPriority(NotificationCompat.PRIORITY_DEFAULT);
        builder.setSmallIcon(R.drawable.ic_launcher_background);

        //builder.setAutoCancel(true);

        NotificationManagerCompat managerCompact = NotificationManagerCompat.from(this);
        managerCompact.notify(1,builder.build());


    }

    private void createNotificationChannel() {
        // Create the NotificationChannel, but only on API 26+ because
        // the NotificationChannel class is new and not in the support library
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            CharSequence name = getString(R.string.channel_name);
            String description = getString(R.string.channel_description);
            int importance = NotificationManager.IMPORTANCE_DEFAULT;
            NotificationChannel channel = new NotificationChannel("Notifikace", name, importance);
            channel.setDescription(description);
            // Register the channel with the system; you can't change the importance
            // or other notification behaviors after this
            NotificationManager notificationManager = getSystemService(NotificationManager.class);
            notificationManager.createNotificationChannel(channel);
        }
    }


    @Override
    public void onLocationChanged(Location location) {
        if (location != null) {
            Log.e("Location Changed", location.getLatitude() + " and " + location.getLongitude());
            mLocationManager.removeUpdates(this);
        }
    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {

    }

    @Override
    public void onProviderEnabled(String s) {

    }

    @Override
    public void onProviderDisabled(String s) {

    }




}